import Acquisition
import Globals
import OFS.SimpleItem
import AccessControl
import OFS.PropertyManager
import OFS.ObjectManager
import webdav.Collection
import Products.XMLDocument
from Globals import HTMLFile
from DateTime import DateTime
import urllib
import string, types
from os.path import join

from xml.dom.utils import FileReader, strip_whitespace

Views_Dir=join(INSTANCE_HOME,'lib','python', 'Products', 'NewsSyndicate', 'Views')
add_ChannelForm=HTMLFile(join('Views', 'channelAdd'),globals())



def add_Channel(self, id, url=None, dc_info=None, REQUEST=None):
	"""
	Add a Channel object to a folder.
	"""

	c=dcChannel(id, url, dc_info)
	self._setObject(id, c)
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class Channel(OFS.ObjectManager.ObjectManager,
				OFS.SimpleItem.Item,
				webdav.Collection.Collection,
				Globals.Persistent,
				Acquisition.Implicit,
				AccessControl.Role.RoleManager,
				OFS.PropertyManager.PropertyManager):
	"""
	Covering RSS 0.90, 0.91, 1.0

	Represents a syndicated channel as defined by RSS 0.91	
	http://www.egroups.com/files/rss-dev/specification.html#
	exceptions:
	    v0.91 exceptions +
	    inchannel ignored - 

	Represents a syndicated channel as defined by RSS 0.91	
	See http://my.netscape.com/publish/help/mnn20/quickstart.html
	exceptions:
	    for security image attrs are static defined as 88x31
	    PICS ranking ignored
	    channel encoding ignored
	    channel language ignored
	    image description ignored
	    textinput ignored

	Represents a syndicated channel as defined by RSS 0.90
	See http://my.netscape.com/publish/help/mnn20/quickstart.html
	
	
	"""
	
	_properties=(
		{'id':'url', 'type':'string', 'mode':'w'},
		{'id':'title', 'type':'string', 'mode':'w'},
		{'id':'link', 'type':'string', 'mode':'w'},
		{'id':'description', 'type':'text', 'mode':'w'},
		{'id':'dc_title', 'type':'string', 'mode':'w'},
		{'id':'dc_subject', 'type':'tokens', 'mode':'w'},
		{'id':'dc_description', 'type':'text', 'mode':'w'},
		{'id':'dc_language', 'type':'string', 'mode':'w'},
		{'id':'ocs_updateFrequency', 'type':'int', 'mode':'w'},
		{'id':'ocs_updatePeriod', 'type':'string', 'mode':'w'}
	)

	manage_options=(
		OFS.ObjectManager.ObjectManager.manage_options+		
		(
		{'label':'Properties', 'action':'manage_propertiesForm'},
		{'label':'Status', 'action':'statusForm'},
		{'label':'View', 'action':'index_html'},
		{'label':'Security', 'action':'manage_access'},
		)
	)

	__ac_permissions__=(
		('View', ('asRSS','asHTML', '', 'index_html')),
		('View management screens', ('manage_propertiesForm', 'statusForm')),
		('Change News Channels', ('manage_properties', 'update'))
	)

	meta_type='News Channel'

	# last update time
	update_last=0
	
	# maximum number of items to show
	max_display=20
	
	def __init__(self, id, url=None, dc_info=None):

		if dc_info is None:
			if url is None:
				raise 'URL not Given'
			else:
				self.url = url
				dc_info = {}

		self.setup_properties(dc_info)
		self.id=id
		self.exists = 0 # exists after first content update - flag for cataloging
		self.exists = self.update() # tech. we don't exist till we're rooted in the ZODB. 

	def setup_properties(self, dc_info):
		info = dc_info.keys()
		
		if 'url' in info:
			self.url = dc_info['url']
		if 'dc:title' in info:
			self.dc_title = dc_info['dc:title']
		else:
			self.dc_title = ''
		if 'dc:subject' in info:
			if type(dc_info['dc:subject']) == types.ListType:
				self.dc_subject = dc_info['dc:subject']
			else:
				self.dc_subject = string.split(dc_info['dc:subject'], ',')
		else:
			self.dc_subject = ''
		if 'dc:description' in info:
			self.dc_description = dc_info['dc:description']
		else:
			self.dc_description = ''
		if 'dc:language' in info:
			self.dc_language = dc_info['dc:language']
		else:
			self.dc_language = ''
		if 'ocs:updateFrequency' in info:
			self.ocs_updateFrequency = dc_info['ocs:updateFrequency']
		else:
			self.ocs_updateFrequency=0
			
		if 'ocs:updatePeriod' in info:
			self.ocs_updatePeriod = dc_info['ocs:updatePeriod']
		else:
			self.ocs_updatePeriod = ''
			
		self.im_title = ''		
		self.im_url = ''
		self.im_link = ''
		self.im_height = '88'
		self.im_width = '31'
		self.type = ''

	def stale(self):
		"""
		Is the content of the channel stale? acquires setting from NewsCenter
		"""				
		if DateTime().timeTime() - self.update_last.timeTime() > (self.default_channel_update_hrs*60*60):
			return 1


	def catalog_unindex(self):
		''' uncatalog channel from catalog '''
		self.news_catalog.uncatalog_object(self.absolute_url())
		for item in self.objectValues():
			self.news_catalog.uncatalog_object(item.absolute_url())

		return 1
	
	def catalog_index(self):
		''' insert self into catalog '''

		try:
			self.news_catalog.catalog_object(self, self.absolute_url())
		except:
			pass
		for item in self.objectValues():
			if 1:
			#try:
				self.news_catalog.catalog_object(item, item.absolute_url())
			#except:
			#	pass
		return 1


	def dump_dc_info(self):
		''' returns a dict of dublin core info. support publishing workflow'''
		# return self.propertyMap
		return {'url':self.url, 'dc:title':self.dc_title, 'dc:subject':self.dc_subject,
			'dc:description':self.dc_description, 'dc:language':self.dc_language,
			'ocs:updateFrequency':self.ocs_updateFrequency, 'ocs:updatePeriod':self.ocs_updatePeriod}


	def get_syndicated_content(self, date):
		""" returns the items within a channel that are newer than the given date """

		content = []
		
		for item in self.objectValues():
			if item.bobobase_modification_time > date:
				content.append(item)
		return item
		
		
	def update(self, REQUEST=None):
		"""
		Fetch the content from a remote url and rebuild
		the channel.

		there is some hacking going on because i haven't determined the correct behavior
		for a channel that raises an error on update. currently a channel that already in existence
		continues as before and update errors are ignored. an alert property would be nice.
		channels that fail update on creation are dropped although this logic takes place in
		the newscenter.		
		
		"""

		try:
			# retrieve RSS file
			fh=urllib.urlopen(self.url)
			fr=FileReader()
			fr.readFile('foo.xml', fh)
			strip_whitespace(fr.document)
			d=fr.document
			fh.close()			
		except:
			''' if we raise an error before we
			exist we should get deleted, else
			noted or silently ignored.'''
			#if not self.exists:
			#raise "Update Error"
			return 
		

		# clear out the old news contents
		if self.exists:
 			self.catalog_unindex()
		
		# get channel attributes
		c=d.getElementsByTagName('channel')[0]

		for cn in c.get_childNodes(): 
			nn = cn.get_nodeName()
			#print nn
			if nn == 'title':
				self.title = cn.get_firstChild().get_nodeValue()
			if nn == 'link':
				self.link = cn.get_firstChild().get_nodeValue()
			if nn == 'description':
				self.description = cn.get_firstChild().get_nodeValue()
			if nn == 'image':
				#print "image info"
				for gcn in cn.get_childNodes(): #grand_child_nodes
					gnn = gcn.get_nodeName()
					#print gcn, gnn
					if gnn == 'title':
						self.im_title = gcn.get_firstChild().get_nodeValue()
					if gnn == 'link':
						self.im_link = gcn.get_firstChild().get_nodeValue()
					if gnn == 'url':
						self.im_url = gcn.get_firstChild().get_nodeValue()						
					# to be safe we'll force the image to default 88x31
					
		# get channel items
		items=d.getElementsByTagName('item')
		for item in items:
			#print item
			# get information about the item
			imap = {} # multiformat hack
			for info in item.get_childNodes():
				if info.get_nodeName() == 'title':
					imap['title'] = info.get_firstChild().get_nodeValue()
				if info.get_nodeName() == 'link':
					imap['link'] = info.get_firstChild().get_nodeValue()
				if info.get_nodeName() == 'description':
					imap['description'] = info.get_firstChild().get_nodeValue()
					
			for required in ['title', 'link', 'description']:
				if not imap.has_key(required):
					imap[required] = ''
			
			# add the item
			self.add_NewsItem(imap['title'], imap['link'], imap['description'])
		
		self.update_last=DateTime()

		
		# update the catalog -- only after we're added to storage
		if self.exists: 
			self.catalog_index()
		
		if REQUEST is not None:
			return self.statusForm(self, REQUEST)
		else:
			return 1

	def manage_beforeDelete(self):
		try:
			self.catalog_unindex()
		except:
			pass
		
	def add_NewsItem(self, title, link, description):
		"""
		Adds an item to the channel. Takes care of avoiding duplicate
		items. Returns the added item's id or None if the item would
		have been a duplicate.
		"""
		
		# clean up item attributes
		title=string.strip(title)
		link=string.strip(link)
		description=string.strip(description)
		
		# check for duplicates
		# note: this could be much more effecient
		for item in self.objectValues('NewsItem'):
			if item.title==title and item.link==link and item.description==description:
				# item is a duplicate
				return
		
		# get a unique id
		id="i%d" % len(self.objectIds()) 
		
		# create and add the item
		item=NewsItem(id, title, link, description)
		return self._setObject(id, item)
	
	statusForm=HTMLFile(join('Views','channelStatus'), globals())
	asRSS=HTMLFile(join(Views_Dir,'channelRSS'), globals())
	
	asHTML=HTMLFile('channelHTML', globals())
	index_html=HTMLFile('channelView', globals())
	slashdot_view=HTMLFile('slashdotView', globals())
	#headline_view=HTMLFile('headlineView', globals())
	
	# Indexing methods
	# ----------------
	
	def SearchableText(self):
		"""
		A concatination of searchable text.
		"""
		return "%s %s" % (self.title, self.description)
		
		
class NewsItem(OFS.SimpleItem.Item,
			Globals.Persistent,
			Acquisition.Implicit,
			AccessControl.Role.RoleManager,
			OFS.PropertyManager.PropertyManager):
	"""
	Represents an item in a syndicated channel.
	"""
	
	meta_type='NewsItem'
	
	_properties=(
		{'id':'title', 'type': 'string', 'mode': 'w'},
		{'id':'link', 'type': 'string', 'mode': 'w'},
		{'id':'description', 'type': 'text', 'mode': 'w'},
	)
	manage_options=(
		{'label':'Properties', 'action':'manage_propertiesForm'},
		{'label':'View', 'action':'index_html'},
		{'label':'Security', 'action':'manage_access'},
	)

	__ac_permissions__=(
		('View', ('', 'index_html')),
		('View management screens', ('manage_propertiesForm',)),
		('Change RSS Channels', ('manage_properties',))
	)
	
	def __init__(self, id, title="", link="", description=""):
		self.id=id
		self.title=title
		self.link=link
		self.description=description
		
	index_html=HTMLFile(join(Views_Dir,'itemView'), globals())

	
	# Indexing methods
	# ----------------
	
	def SearchableText(self):
		"""
		A concatination of searchable text.
		"""
		return "%s %s" % (self.title, self.description)
	
	def Channel(self):
		"""
		Containing channel name.
		"""
		return self.aq_parent.title
	
	def Date(self):
		"""
		Date of the item.
		"""
		return self.bobobase_modification_time()
	

		
		
