
import Acquisition
import Globals
import OFS.SimpleItem
import AccessControl
import OFS.PropertyManager
import OFS.ObjectManager
import webdav.Collection

import Products.BTreeFolder
from Globals import HTMLFile
from DateTime import DateTime
import urllib
import string
from xml.dom.utils import FileReader, strip_whitespace
import cPickle
import Channel
from os.path import join
import time
from Parsers import ocs

Views_Dir=join(INSTANCE_HOME,'lib','python', 'Products', 'NewsSyndicate', 'Views')

add_NewsCenterForm=HTMLFile('NewsCenterAdd', globals())

def add_NewsCenter(self, id, REQUEST=None):
	"""
        Add A NewCenter
	"""
	n=NewsCenter(id)
	self._setObject(id, n)
	
	if REQUEST is not None:
		return self.manage_main(self, REQUEST)


class NewsCenter(OFS.ObjectManager.ObjectManager,
				OFS.SimpleItem.Item,
				webdav.Collection.Collection,
				Globals.Persistent,
				Acquisition.Implicit,
				AccessControl.Role.RoleManager,
				OFS.PropertyManager.PropertyManager):
	"""
        NewsCenter for importing RSS Channels
        
	"""
	
	_properties=(
		{'id':'title' , 'type':'string', 'mode':'w'},
		{'id':'publish_rss_as_acquired', 'type':'int', 'mode':'w'},
		{'id':'update_rss_from_fs', 'type':'int', 'mode':'w'},
		{'id':'update_rss_fs_dir', 'type':'string', 'mode':'w'},
		{'id':'retrieve_list_as_acquired', 'type':'int', 'mode':'w'},
		{'id':'retrieve_list_from_fs', 'type':'int', 'mode':'w'},
		{'id':'retrieve_list_fs_dir', 'type':'string', 'mode':'w'},
		{'id':'default_channel_update_hrs', 'type':'int', 'mode':'w'},
		{'id':'default_list_update_hrs', 'type':'int', 'mode':'w'},
	)
 
	manage_options=(
		OFS.ObjectManager.ObjectManager.manage_options+
		OFS.PropertyManager.PropertyManager.manage_options+
		(
		{'label':'View', 'action':'index_html'},
		{'label':'Channels', 'action':'channel_html'},
		)+
		AccessControl.Role.RoleManager.manage_options+
		OFS.SimpleItem.SimpleItem.manage_options
	)

	__ac_permissions__=(
		('View', ('asHTML', '', 'index_html')),
		('View management screens', ('manage_propertiesForm', 'channel_html')),
		('Change NewsCenter', ('manage_properties', 'update'))
	)

	meta_type='NewsCenter'
	meta_types=({'name':'Channel', 'action':'manage_addChannel'},) 
	def __init__(self,id):
		self.id = id
		self.channel_list = []
		self.r_channel_list = []
		self.setup_properties()
		self.setup_storage_folders()
		self.setup_catalog()
		#self.setup_views() # makes the whole thing unstable

	search_results=HTMLFile('NewsCenterSearchResults', globals())
	index_html=HTMLFile('NewsCenterView', globals())
	channel_html=HTMLFile('NewsCenterChannelManage', globals())
	asHTML=HTMLFile('NewsCenterHTML', globals())
	unpub_channels=HTMLFile('NewsCenterViewUnChannels', globals())
	
	#################################
	# Configuration/Setup Methods
	#################################

	def setup_views(self):
		''' setups site configurable views in the management screens '''

		self.manage_addFolder('views')

		fh = open(join(Views_Dir, 'NewsCenterView.dtml'))
		self.views.manage_addDTMLMethod(id='news_center_index_html', file=fh)
		self.index_html = self.views.news_center_index_html		
		fh.close()

		fh = open(join(Views_Dir, 'NewsCenterHTML.dtml'))
		self.views.manage_addDTMLMethod(id='news_center_as_html', file=fh)
		self.asHTML = self.views.news_center_as_html
		fh.close()

		fh = open(join(Views_Dir, 'NewsCenterSearchResults.dtml'))
		self.views.manage_addDTMLMethod(id='news_center_search_results', file=fh)
		self.search_results = self.views.news_center_search_results
		fh.close()

		fh = open(join(Views_Dir, 'channelView.dtml'))
		self.views.manage_addDTMLMethod(id='channel_index_html', file=fh)
		fh.close()
		
		fh = open(join(Views_Dir, 'channelHTML.dtml'))
		self.views.manage_addDTMLMethod(id='channel_as_html', file=fh)
		fh.close()		

		fh = open(join(Views_Dir, 'slashdotView.dtml'))
		self.views.manage_addDTMLMethod(id='channel_slashdot_view', file=fh)
		fh.close()
		
		return 1
	
	def setup_properties(self):
		''' setups initial property values '''
		self.publish_rss_as_acquired=0
		self.update_rss_from_fs = 0
		self.update_rss_fs_dir = ''
		self.retrieve_list_as_acquired = 1
		self.retrieve_list_fs_dir = ''
		self.retrieve_list_from_fs = 0
		self.default_channel_update_hrs=6
		self.default_list_update_hrs = 24*30
		return 10

	def setup_catalog(self):
		''' setups catalog for searching news channels '''
		
		id = 'news_catalog'
		catalog = Products.ZCatalog.ZCatalog.ZCatalog(id)
		self._setObject(id, catalog)
		
		# setup indexes and such
		self.news_catalog.manage_addColumn('description')

		self.news_catalog.manage_addColumn('Date')
		self.news_catalog.manage_addIndex('Date', 'FieldIndex')
		
		self.news_catalog.manage_addColumn('ChannelName')
		self.news_catalog.manage_addIndex('ChannelName', 'FieldIndex')
		
		self.news_catalog.manage_addColumn('link')
		self.news_catalog.manage_addIndex('link', 'FieldIndex')

		self.news_catalog.manage_addIndex('SearchableText', 'TextIndex')

		# get rid of defaults
		self.news_catalog.manage_delColumns(['bobobase_modification_time', 'summary'])
		self.news_catalog.manage_delIndexes(['bobobase_modification_time', 'PrincipiaSearchSource'])
		
		return 1

	def setup_storage_folders(self):	
		''' configures storage for channels'''
		
		id = 'p_storage'
		btreefolder = Products.BTreeFolder.BTreeFolder.BTreeFolder()
		btreefolder.id = id
		self._setObject(id, btreefolder)
		
		id = 'up_storage'
		btreefolder = Products.BTreeFolder.BTreeFolder.BTreeFolder()
		btreefolder.id = id
		self._setObject(id, btreefolder)
		return 1

	#########*#######################
	# Testing and Debugging
	#################################
	#  removed for distribution

        #################################
	# Indexing Methods
	#################################
	def index_all_channels(self):
		''' guess '''
		for object in self.p_storage.objectValues():
			object.catalog_index()
		return 1

	def index_channel(self, cid):
		''' guess '''
		if hasattr(self.p_storage, cid):
			getattr(self.p_storage, cid).catalog_index()
			return 1
		return 0		

	def unindex_channel(self, cid):
		''' remove channel from catalog '''
		if hasattr(self.p_storage, cid):
			getattr(self.p_storage, cid).catalog_unindex()
			return 1
		return 0

	def unindex_all_channels(self):
		''' remove all channels from catalog '''
		for object in self.p_storage.objectValues():
			object.catalog_unindex()
		return 1
	
	#################################
	# Channel List Methods
	#################################
	# Lists have two states added and retrieved. 
	# retrieved lists are lists that have been parsed for rss channels
	# and they're channels added to the newscenter (either published or un published)
	# added channel lists exist only as a tuple (type, url)

	def get_retrieved_lists(self):
		''' get all retrieved channel lists '''
		return self.r_channel_list

	def delete_list(self, url, type):
		''' delete a list - why? '''
		if self.channel_list.count((type, url)) != 0:
			cl = self.channel_list
			cl.pop(cl.index((type,url)))
			self.channel_list = cl
		elif self.r_channel_list.count((type, url)) !=0:
			rcl = self.r_channel_list
			rcl.pop(rcl.index((type,url)))
			self.channel_list = rcl
		else:
			return 0
		return 1

	def get_added_lists(self):
		''' get all added channel lists '''		
		return self.channel_list

	def update_channel_lists(self):
		''' for this to work we need to insure uniqueness '''
		''' we insure channel uniqueness by hashing hmmmm '''
		''' also need to allow for fs updates '''
		pass
		
	def set_channel_list_retrieve(self, ch_list_index):
		''' moves a channel list from added state to retrieved state '''
		type, url = self.channel_list[ch_list_index]
		self.get_channels_from_list(type, url)
		
		rcl = self.r_channel_list # just to make sure we play nice with persistence - not sure if this is req.
		if rcl.count((type,url)) == 0: # only add unique
			rcl.append((type,url))
			self.r_channel_list = rcl

		cl = self.channel_list
		cl.pop(ch_list_index)
		self.channel_list = cl

	def add_channel_list(self, type, url, force=None):
		''' initial add channel list to the NewsCenter.
		force overides the property settings to set the channel to be retrieved from the url'''

		if self.retrieve_list_as_acquired or force:
			rcl = self.r_channel_list
			if rcl.count((type, url)) == 0 and self.channel_list.count((type, url))==0:
				rcl.append(type, url)
				self.r_channel_list = rcl
				v=self.get_channels_from_list(type, url, force) # gets channels and store channels
				return v

			else:
				return 0
		
		else:
			cl = self.channel_list
			if cl.count((type, url)) == 0 and self.r_channel_list.count((type,url))==0:
				cl.append((type, url))
				self.channel_list = cl
				return 1
			else:
				return 0


	def get_channels_from_list(self, type, url, force):
		''' retrieve rss_channel list from channel list '''

		''' This should get the dom and pass it to appropiate dom_parser. or not since not all
		    the parsers will be operating on a dom ala the meerkat...
		    so we pass it in the appropiate file handle. url or fs
		    '''

		''' added subcommits cause of thrashing concerns '''

		rss_c_list = []
		
		#print 'getting channel', self.retrieve_list_from_fs, force

		stime = time.time()
		
		if self.retrieve_list_from_fs and not force:
			index = string.rfind(url,'/')
			fn = url[index:]
			fh = open(self.retrieve_list_fs_dir+fn)
			if fh:
				rss_c_list = eval(type+'.get_rss_channel_info(fh)')
		else:
			fh = urllib.urlopen(url)
			#print "retrieving ocs list "
			if fh: 
				rss_c_list = eval(type+'.get_rss_channel_info(fh)')
			
		if self.publish_rss_as_acquired:
			# get rss and store			
			#print ' adding channels ', self.publish_rss_as_acquired
			counter = 0
			for dc_info in rss_c_list:
				counter = counter +1
				self.add_p_rss_channel(dc_info)
				if counter%100==0:
					get_transaction().commit(1)
			return "%d channels found in %d seconds"%(counter, time.time()-stime)				

		else:
			#print ' adding channel stores ', self.publish_rss_as_acquired
			#print len(rss_c_list)					
			counter = 0
			for dc_info in rss_c_list:
				counter = counter +1
				self.add_up_rss_channel(dc_info)
				#del dc_info
				if counter%100==0:
					get_transaction().commit(1)
				
			return "%d channels found in %d seconds"%(counter, time.time()-stime)
			
	#################################
	# RSS channel Methods
	#################################
	# i'm not sure about the best way to insure channel uniqueness.
	# for now I just use abs(hash(url)) as an id.
	# i need to do security checks on the eval stuff
	
	def add_p_rss_channel(self, dc_info):
		''' add a channel to publish store '''
		#id="i%d" % len(self.p_storage.objectIds())
		
		id = "i%d" % abs(hash(dc_info['url']))

		if self.p_storage._tree.has_key(id)==1:
			return

		# create and add the item
		
		#if 1:
		try:
			item=Channel.Channel(id, None, dc_info)
		except:
			item=None
			
		if item is not None and item.exists:
			self.p_storage._setObject(id, item)
			getattr(self.p_storage, id).catalog_index() # explicit... 			
			return 1
		return 0


	def publish_all_rss_channels(self):
		''' publishes all unpublished rss channels '''
		id_list = self.up_storage.objectIds()
		return self.publish_rss_channels(id_list)
	
	def publish_rss_channels(self, channel_ids=None, ids=None):
		''' move a channel from unpublished store to published - creates the channel'''

		# management screen stuff
		if channel_ids: pass
		elif ids: channel_ids = ids
		else: return "No Channels Selected"
			
		counter = 0
		stime = time.time()
		for id in channel_ids:
			if hasattr(self.up_storage, id):
				dc_info = getattr(self.up_storage, id).dc_info
				self.add_p_rss_channel(dc_info)
				self.up_storage._delObject(id)
				self.index_channel(id)
				counter = counter + 1
		#print counter
		return "%d channels published in %d seconds"%(counter, time.time()-stime)


	def unpublish_rss_channels(self, channel_ids=None, ids=None):
		''' move a channel from published store to unpublished '''

		# management screen stuff
		if channel_ids: pass
		elif ids: channel_ids = ids
		else: return "No Channels Selected"

		stime = time.time()
		counter = 0
		for id in channel_ids:
			if hasattr(self.p_storage, id):
				channel=getattr(self.p_storage, id)
				dc_info = channel.dump_dc_info()
				channel.catalog_unindex() # if i put this in manage_beforeDel i think i don't need this
				self.add_up_rss_channel(dc_info)
				self.p_storage._delObject(id)
				counter = counter+1
		return "%d channels unpublished in %d seconds"%(counter, time.time()-stime)				
	
	def add_up_rss_channel(self, dc_info):
		''' add a channel to unpublished store '''

		id = "i%d" % abs(hash(dc_info['url']))

		# uniqueness -- ugly breaks down encaps. 3am hack
		if self.up_storage._tree.has_key(id)==1:
			return -1		

		ch_store = StorageItem()
		ch_store.dc_info = dc_info
		id = "i%d" % abs(hash(dc_info['url']))		
		self.up_storage._setObject(id, ch_store)
		return 1


	def delete_all_channels(self):
		''' channel geddon '''
		pass		

	def delete_rss_channels(self, cids=None, ids=None): 
		''' check for channel in published and unpublished and remove it '''
		''' probably needs to target a specific state'''

		# management screen stuff
		if channel_ids: pass
		elif ids: channel_ids = ids
		else: return "No Channels Selected"
		
		counter = 0
		for id in cids:
			if hasattr(self.p_storage, id):
				counter = counter +1
				channel = getattr(self.p_storage, id)
				channel.catalog_unindex()				
				self.p_storage._delObject(cid)
			elif hasttr(self.up_storage, id):
				counter = counter +1
				self.up_storage._delObject(cid)

		return counter
	
	#################################
	# Accessor Method 
	#################################
	def get_channel(self, cid=None, ids=None):
		''' gets a channel '''

		# management screen stuff
		if channel_ids: pass
		elif ids: channel_ids = ids
		else: return "No Channels Selected"		
		
		if hasattr(self.p_storage, cid):
			return getattr(self.p_storage, cid)
		else:
			return "one at a time"

	def get_channels(self, cids):
		cl = []
		for cid in cids:
			cl.append(self.get_channel(cid))
		return cl
			
	#################################
	# Email Syndication Methods
	#################################

	"""
	There are too many formats for user storage/types 
	so instead of these we provide the 
	hook for a site to build their own mail syndication app.
	"""
	def get_syndicated_content(channelids, lastdate=None):
		""" returns list of news items for given channel list that have been added
		since lastdate """
		
		new_content = []

		if not lastdate: # if no date given give new stories from the last 24hrs. 
			lastdate = DateTime()-1
		
		for cid in channelids:
			if hasattr(p_storage, cid):
				new_content.extend(getattr(p_storage, cid).get_syndicated_content(lastdate))
				
		return new_content


class StorageItem(OFS.SimpleItem.SimpleItem, Globals.Persistent):
	pass


